//
// aniUI.h
//
// The header file for the UI part
//

#ifndef __aniUI_h__
#define __aniUI_h__

#pragma warning(disable:4786)

#include <FL/Fl.H>
#include <FL/Fl_Window.H>
#include <FL/Fl_Menu_Bar.H>
#include <FL/Fl_Value_Slider.H>
#include <FL/Fl_Button.H>

#include <FL/fl_file_chooser.H>		// FLTK file chooser

#include "modeler.h"

#include "glView.h"

#define CONTROL_MODE	0
#define CURVE_MODE		1

class CAniDoc;
class fltkCurveEditor;

class CAniUI {
public:
	CAniUI(); 
	~CAniUI();

	// The FLTK widgets
	Fl_Window*			m_mainWindow;
	Fl_Menu_Bar*		m_menubar;

	// member functions
	void show();

	CAniDoc*	getDocument();
	void		setDocument(CAniDoc*);

	int			getDrawMode() { return m_nDarwMode; };

	void		load_named_object(CAniUI* pUI, CAniDoc* pDoc, char* libname);
	void		load_named_ani(char* ani);

	void		addView( CGLView* view )
				{ glViews.push_back( view ); }

	void		removeView( CGLView* view )
				{ glViews.remove( view ); }

	void		addPanel( ctrlPanel* panel )
				{ panels.push_back( panel ); }

	void		removePanel( ctrlPanel* panel )
				{ panels.remove( panel ); }

	void		refreshAll();

	void		closeAllViews();
	void		closeAllPanels();

	fltkCurveEditor *m_pCurveEditor;

private:
	CAniDoc*	m_pDoc;

	int			m_nDarwMode;
	int			m_nOpMode;

	list<CGLView*>		glViews;
	list<ctrlPanel*>	panels;

// static class members
	static Fl_Menu_Item menuitems[];

	static CAniUI* whoami(Fl_Menu_* o);

	static void cb_load_object(Fl_Menu_* o, void* v);

	static void cb_load_scene(Fl_Menu_* o, void* v);
	static void cb_close_scene(Fl_Menu_* o, void* v);

	static void cb_load_animation(Fl_Menu_* o, void* v);
	static void cb_save_animation(Fl_Menu_* o, void* v);

	static void cb_save_snapshot(Fl_Menu_* o, void* v);
	static void cb_save_ray_snapshot(Fl_Menu_* o, void* v);
	static void cb_save_movie(Fl_Menu_* o, void* v);
	static void cb_save_ray_movie(Fl_Menu_* o, void* v);

	static void cb_redraw(Fl_Menu_* o, void* v);
	static void cb_set_opmode(Fl_Menu_* o, void* v);

	static void cb_set_drawmode(Fl_Menu_* o, void* v);
	static void cb_set_drawquality(Fl_Menu_* o, void* v);

	static void CAniUI::cb_fps(Fl_Menu_* o, void* v);
	static void CAniUI::cb_reset_time(Fl_Menu_* o, void* v);
	static void CAniUI::cb_view_curve(Fl_Menu_* o, void* v);
	static void CAniUI::cb_view_bezier(Fl_Menu_* o, void* v);
	static void CAniUI::cb_set_viewpointEdit(Fl_Menu_* o, void* v);

	static void cb_exit(Fl_Menu_* o, void* v);
	static void cb_about(Fl_Menu_* o, void* v);

	static void cb_exit2(Fl_Widget* o, void* v);
};

#endif
