//
// scene.h
//
// The Scene class
//

#ifndef __SCENE_H__
#define __SCENE_H__

#pragma warning(disable:4786)

#include <list>
#include <algorithm>

using namespace std;

#include <Fl/Fl_Window.H>
#include <Fl/Fl_Button.H>
#include <Fl/Fl_Light_Button.H>
#include <Fl/Fl_Check_Button.H>
#include <Fl/Fl_Box.H>
#include <Fl/Fl_Group.H>

#include "object.h"
#include "arcball.h"

class CAniDoc;
class CGLView;
enum State {NONE=0,ROTATE,DOLLY,ZOOM, CONSTRAIN_ABOUT_Y, CONSTRAIN_ABOUT_X, CONSTRAIN_ABOUT_Z};

const string qnowx("Quaternion X#0@101104");
const int qnowxid = 101104;
const string qnowy("Quaternion Y#0@101115");
const int qnowyid = 101115;
const string zoomed("Zoom#0@137113");
const int zoomid = 137113;
const string qnowz("Quaternion Z#0@111103");
const int qnowzid = 111103;
const string qnoww("Quaternion W#0@113123");
const int qnowwid = 113123;
const string centerz("Translate Z#0@137101");
const int centerzid = 137101;
const string centerx("Translate X#0@123123");
const int centerxid = 123123;
const string centery("Translate Y#0@137041");
const int centeryid = 137041;
const string viewpoint("Viewpoint#0@41041");
const int viewpointid = 41041;

class scene_params {
public:
	scene_params(void *p, Object *o) { self=p, obj=o; };
	void *self;
	Object* obj;
};

class Scene {
public:
	typedef list<Object*>::iterator			giter;
	typedef list<Object*>::const_iterator	cgiter;

public:
	Scene() : objects() { m_nObjCnt=0; m_pView=NULL; m_pObjList=NULL; m_pActiveObj=NULL; };
	Scene(Object* obj);
	~Scene();

	void add( Object* obj )
	{ objects.push_back( obj ); m_nObjCnt++; obj->setScene(this); }
	
	void redraw(int drawMode);

	void refresh();

	void setDocument(CAniDoc* pDoc) { m_pDoc=pDoc;};
	CAniDoc *getDocument() { return m_pDoc; };

	void draw_mode_setup(int drawMode);
	void draw_mode_finish(int drawMode);

	void show();

	void setView(CGLView *view) { m_pView=view; };
	CGLView *getView() { return m_pView; };

	void createView();
	void createObjList();

	void saveImage(char *iname);
	void draw_circle(double x, double y, double radius);

	Fl_Window *m_pObjList;

	Object *m_pActiveObj;
	Vec3 center;
	Vec3 orig_center;
	double radius;

	bool dragging;
	bool viewpointEdit;
	State current;
	Vec3 vfrom;
	Vec3 vto;
	Quat qdown;
	Quat qnow;
	Mat4 model;
	double zoom, orig_zoom;
	int constrained_rot;

private:
    list<Object*> objects;
	CGLView *m_pView;

	int m_nObjCnt;
	CAniDoc *m_pDoc;

	static void cb_fork_object(Fl_Widget* o, void* v);
	static void cb_toggle_object(Fl_Widget* o, void* v);
	static void cb_activate_object(Fl_Widget* o, void* v);
};

#endif