// disable warning message about converting double constant to float constant
#pragma warning(disable:4305)

#include <modelerdll.h>

#define BASE_ROTATION   1
#define LOWER_TILT      2
#define UPPER_TILT      3
#define UPPER_TWIST     4
#define BASE_LENGTH		5
#define LOWER_LENGTH	6
#define UPPER_LENGTH	7

MODEL("classarm");

void base(float h);
void lower_arm(float h);
void upper_arm(float h);
void y_box(float h);

bool init(void* args)
{
    scale( BASE_ROTATION, "base rotation (theta)", -180.0, 180.0, 0.1, 0.0 );
    scale( LOWER_TILT, "lower arm tilt (phi)", -90.0, 90.0, 0.1, 0.0 );
    scale( UPPER_TILT, "upper arm tilt (psi)", -90.0, 90.0, 0.1, 0.0 );
    scale( BASE_LENGTH, "base height (h1)", 1, 10.0, 0.1, 2.0 );
    scale( LOWER_LENGTH, "lower arm length (h2)", 1, 10.0, 0.1, 3.0 );
    scale( UPPER_LENGTH, "upper arm length (h3)", 1, 10.0, 0.1, 1.0 );
    
    return true;
}
    

void redraw(int drawmode)
{

	static GLfloat lmodel_ambient[] = {0.4,0.4,0.4,1.0};

    glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT );
	glEnable(GL_LIGHTING);
	glLightModelfv(GL_LIGHT_MODEL_AMBIENT,lmodel_ambient);	// add some ambience

	/* pick up the slider values */

	float theta = get_control_f( BASE_ROTATION );
	float phi = get_control_f( LOWER_TILT );
	float psi = get_control_f( UPPER_TILT );
	float h1 = get_control_f( BASE_LENGTH );
	float h2 = get_control_f( LOWER_LENGTH );
	float h3 = get_control_f( UPPER_LENGTH );

    // define the viewing transform

    glMatrixMode( GL_MODELVIEW );
//    glLoadIdentity();
//    glTranslatef( 0.0, -4.0, -20.0 );		// push it away from us so we can see it
	glTranslatef( 0.0, -4.0, 0.0 );

	// define the model

    glRotatef( theta, 0.0, 1.0, 0.0 );		// turn the whole assembly around the y-axis. 
	base(h1);								// draw the base

    glTranslatef( 0.0, h1, 0.0 );			// move to the top of the base
    glRotatef( phi, 0.0, 0.0, 1.0 );		// rotate around the z-axis for the lower arm
	lower_arm(h2);							// draw the lower arm

    glTranslatef( 0.0, h2, 0.0 );			// move to the top of the lower arm
    glRotatef( psi, 0.0, 0.0, 1.0 );		// rotate  around z-axis for the upper arm
	upper_arm(h3);							// draw the upper arm
}

void base(float h) {
	diffuse_color( 0.8, 0.6, 0.6 );
	ambient_color( 0.8, 0.6, 0.6 );
	glPushMatrix();
	glRotatef(-90,1.0,0.0,0.0);				// flip it up so we can draw along z
	cylinder( h, 1.0, 1.0 );
	glPopMatrix();
}

void lower_arm(float h) {					// draw the lower arm
	diffuse_color( 0.6, 0.8, 0.6 );
	ambient_color( 0.6, 0.8, 0.6 );
	y_box(h);
}

void upper_arm(float h) {					// draw the upper arm
	diffuse_color( 0.6, 0.6, 0.8 );
	ambient_color( 0.6, 0.6, 0.8 );
	y_box(h);
}

void y_box(float h) {

	glBegin( GL_QUADS );

	glNormal3d( 1.0 ,0.0, 0.0);			// +x side
	glVertex3d( 0.25,0.0, 0.25);
	glVertex3d( 0.25,0.0,-0.25);
	glVertex3d( 0.25,  h,-0.25);
	glVertex3d( 0.25,  h, 0.25);

	glNormal3d( 0.0 ,0.0, -1.0);		// -z side
	glVertex3d( 0.25,0.0,-0.25);
	glVertex3d(-0.25,0.0,-0.25);
	glVertex3d(-0.25,  h,-0.25);
	glVertex3d( 0.25,  h,-0.25);

	glNormal3d(-1.0, 0.0, 0.0);			// -x side
	glVertex3d(-0.25,0.0,-0.25);
	glVertex3d(-0.25,0.0, 0.25);
	glVertex3d(-0.25,  h, 0.25);
	glVertex3d(-0.25,  h,-0.25);

	glNormal3d( 0.0, 0.0, 1.0);			// +z side
	glVertex3d(-0.25,0.0, 0.25);
	glVertex3d( 0.25,0.0, 0.25);
	glVertex3d( 0.25,  h, 0.25);
	glVertex3d(-0.25,  h, 0.25);

	glNormal3d( 0.0, 1.0, 0.0);			// top (+y)
	glVertex3d( 0.25,  h, 0.25);
	glVertex3d( 0.25,  h,-0.25);
	glVertex3d(-0.25,  h,-0.25);
	glVertex3d(-0.25,  h, 0.25);

	glNormal3d( 0.0,-1.0, 0.0);			// bottom (-y)
	glVertex3d( 0.25,0.0, 0.25);
	glVertex3d(-0.25,0.0, 0.25);
	glVertex3d(-0.25,0.0,-0.25);
	glVertex3d( 0.25,0.0,-0.25);

	glEnd();
}

